@@ -22,7 +22,7 @@ import (
 	awsprovider "sigs.k8s.io/cluster-api-provider-aws/pkg/apis/awsprovider/v1beta1"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 
-	machineapi "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
+	machineapi "github.com/openshift/api/machine/v1beta1"
 
 	"github.com/openshift/hive/apis"
 	hivev1 "github.com/openshift/hive/apis/hive/v1"
@@ -31,6 +31,10 @@ import (
 	controllerutils "github.com/openshift/hive/pkg/controller/utils"
 )
 
+const (
+	fakeKMSKeyARN = "fakearn"
+)
+
 func TestAWSActuator(t *testing.T) {
 	tests := []struct {
 		name                         string
@@ -42,6 +46,7 @@ func TestAWSActuator(t *testing.T) {
 		expectedSubnetIDInMachineSet bool
 		expectedErr                  bool
 		expectedCondition            *hivev1.MachinePoolCondition
+		expectedKMSKey               string
 	}{
 		{
 			name:              "generate single machineset for single zone",
@@ -240,6 +245,72 @@ func TestAWSActuator(t *testing.T) {
 				Reason: "InsufficientPublicSubnets",
 			},
 		},
+		{
+			name:              "public subnets all don't have route tables pointing to igw",
+			clusterDeployment: testClusterDeployment(),
+			poolName:          testMachinePool().Name,
+			existing: []runtime.Object{
+				func() *hivev1.MachinePool {
+					pool := testMachinePool()
+					pool.Spec.Platform.AWS.Zones = []string{"zone1", "zone2"}
+					pool.Spec.Platform.AWS.Subnets = []string{"subnet-zone1", "subnet-zone2", "pubSubnet-zone1", "pubSubnet-zone2"}
+					return pool
+				}(),
+			},
+			mockAWSClient: func(client *mockaws.MockClient) {
+				mockDescribeSubnets(client, []string{"zone1", "zone2"},
+					[]string{"subnet-zone1", "subnet-zone2"}, []string{"pubSubnet-zone1", "pubSubnet-zone2"}, "vpc-1")
+				mockDescribeRouteTables(client, map[string]bool{
+					"subnet-zone1":    false,
+					"subnet-zone2":    false,
+					"pubSubnet-zone1": false,
+					"pubSubnet-zone2": false,
+				}, "vpc-1")
+			},
+			expectedCondition: &hivev1.MachinePoolCondition{
+				Type:   hivev1.InvalidSubnetsMachinePoolCondition,
+				Status: corev1.ConditionFalse,
+				Reason: "ValidSubnets",
+			},
+			expectedMachineSetReplicas: map[string]int64{
+				generateAWSMachineSetName("zone1"): 2,
+				generateAWSMachineSetName("zone2"): 1,
+			},
+			expectedSubnetIDInMachineSet: true,
+		},
+		{
+			name:              "public subnets some don't have route tables pointing to igw",
+			clusterDeployment: testClusterDeployment(),
+			poolName:          testMachinePool().Name,
+			existing: []runtime.Object{
+				func() *hivev1.MachinePool {
+					pool := testMachinePool()
+					pool.Spec.Platform.AWS.Zones = []string{"zone1", "zone2"}
+					pool.Spec.Platform.AWS.Subnets = []string{"subnet-zone1", "subnet-zone2", "pubSubnet-zone1", "pubSubnet-zone2"}
+					return pool
+				}(),
+			},
+			mockAWSClient: func(client *mockaws.MockClient) {
+				mockDescribeSubnets(client, []string{"zone1", "zone2"},
+					[]string{"subnet-zone1", "subnet-zone2"}, []string{"pubSubnet-zone1", "pubSubnet-zone2"}, "vpc-1")
+				mockDescribeRouteTables(client, map[string]bool{
+					"subnet-zone1":    false,
+					"subnet-zone2":    false,
+					"pubSubnet-zone1": true,
+					"pubSubnet-zone2": false,
+				}, "vpc-1")
+			},
+			expectedCondition: &hivev1.MachinePoolCondition{
+				Type:   hivev1.InvalidSubnetsMachinePoolCondition,
+				Status: corev1.ConditionFalse,
+				Reason: "ValidSubnets",
+			},
+			expectedMachineSetReplicas: map[string]int64{
+				generateAWSMachineSetName("zone1"): 2,
+				generateAWSMachineSetName("zone2"): 1,
+			},
+			expectedSubnetIDInMachineSet: true,
+		},
 		{
 			name:              "supported spot market options",
 			clusterDeployment: withClusterVersion(testClusterDeployment(), "4.5.0"),
@@ -267,6 +338,21 @@ func TestAWSActuator(t *testing.T) {
 				Reason: "UnsupportedSpotMarketOptions",
 			},
 		},
+		{
+			name:              "kms key disk encryption",
+			clusterDeployment: withClusterVersion(testClusterDeployment(), "4.5.0"),
+			poolName:          testMachinePool().Name,
+			existing: []runtime.Object{
+				withKMSKey(testMachinePool()),
+			},
+			mockAWSClient: func(client *mockaws.MockClient) {
+				mockDescribeAvailabilityZones(client, []string{"zone1"})
+			},
+			expectedMachineSetReplicas: map[string]int64{
+				generateAWSMachineSetName("zone1"): 3,
+			},
+			expectedKMSKey: fakeKMSKeyARN,
+		},
 		{
 			name:              "unsupported configuration condition cleared",
 			clusterDeployment: withClusterVersion(testClusterDeployment(), "4.4.0"),
@@ -339,7 +425,7 @@ func TestAWSActuator(t *testing.T) {
 			if test.expectedErr {
 				assert.Error(t, err, "expected error for test case")
 			} else {
-				validateAWSMachineSets(t, generatedMachineSets, test.expectedMachineSetReplicas, test.expectedSubnetIDInMachineSet)
+				validateAWSMachineSets(t, generatedMachineSets, test.expectedMachineSetReplicas, test.expectedSubnetIDInMachineSet, test.expectedKMSKey)
 			}
 			if test.expectedCondition != nil {
 				cond := controllerutils.FindMachinePoolCondition(pool.Status.Conditions, test.expectedCondition.Type)
@@ -375,7 +461,7 @@ func TestGetAWSAMIID(t *testing.T) {
 	for _, tc := range cases {
 		t.Run(tc.name, func(t *testing.T) {
 			scheme := runtime.NewScheme()
-			machineapi.SchemeBuilder.AddToScheme(scheme)
+			machineapi.AddToScheme(scheme)
 			awsprovider.SchemeBuilder.AddToScheme(scheme)
 			actualAMIID, actualErr := getAWSAMIID(tc.masterMachine, scheme, log.StandardLogger())
 			if tc.expectError {
@@ -389,13 +475,13 @@ func TestGetAWSAMIID(t *testing.T) {
 	}
 }
 
-func validateAWSMachineSets(t *testing.T, mSets []*machineapi.MachineSet, expectedMSReplicas map[string]int64, expectedSubnetID bool) {
+func validateAWSMachineSets(t *testing.T, mSets []*machineapi.MachineSet, expectedMSReplicas map[string]int64, expectedSubnetID bool, expectedKMSKey string) {
 	assert.Equal(t, len(expectedMSReplicas), len(mSets), "different number of machine sets generated than expected")
 
 	for _, ms := range mSets {
 		expectedReplicas, ok := expectedMSReplicas[ms.Name]
-		if assert.True(t, ok, "unexpected machine set") {
-			assert.Equal(t, expectedReplicas, int64(*ms.Spec.Replicas), "replica mismatch")
+		if assert.True(t, ok, "unexpected machine set", ms.Name) {
+			assert.Equal(t, expectedReplicas, int64(*ms.Spec.Replicas), "replica mismatch", ms.Name)
 		}
 
 		awsProvider, ok := ms.Spec.Template.Spec.ProviderSpec.Value.Object.(*awsprovider.AWSMachineProviderConfig)
@@ -407,6 +493,8 @@ func validateAWSMachineSets(t *testing.T, mSets []*machineapi.MachineSet, expect
 			assert.Equal(t, testAMI, *awsProvider.AMI.ID, "unexpected AMI ID")
 		}
 
+		assert.Equal(t, expectedKMSKey, *awsProvider.BlockDevices[0].EBS.KMSKey.ARN)
+
 		if expectedSubnetID {
 			providerConfig := ms.Spec.Template.Spec.ProviderSpec.Value.Object.(*awsprovider.AWSMachineProviderConfig)
 			assert.NotNil(t, providerConfig.Subnet.ID, "missing subnet ID")
@@ -458,6 +546,10 @@ func mockDescribeSubnets(client *mockaws.MockClient, zones []string, privateSubn
 			SubnetId:         &pubSubnetIDs[i],
 			AvailabilityZone: &zones[i],
 			VpcId:            &vpcID,
+			Tags: []*ec2.Tag{{
+				Key:   aws.String(tagNameSubnetPublicELB),
+				Value: aws.String("1"),
+			}},
 		}
 	}
 	output := &ec2.DescribeSubnetsOutput{
@@ -551,3 +643,8 @@ func withSpotMarketOptions(pool *hivev1.MachinePool) *hivev1.MachinePool {
 	pool.Spec.Platform.AWS.SpotMarketOptions = &awshivev1.SpotMarketOptions{}
 	return pool
 }
+
+func withKMSKey(pool *hivev1.MachinePool) *hivev1.MachinePool {
+	pool.Spec.Platform.AWS.EC2RootVolume.KMSKeyARN = fakeKMSKeyARN
+	return pool
+}
